function [PICT,PSIICTmin,PSICOLL] = psiFilterICT(P,PSICOLL)
%%PSIFILTERICT constructs the interconnection between P and PSICOLL.
% Input arguments:
% (1) P is the nominal plant.
% (2) PSICOLL is a cell array containing all PSI filters to be appended.
% Output arguments:
% (1) PICT is the interconnection between P and PSIICT.
% (2) PSIICTmin is the interconnection between all the PSI filters.
% (3) PSICOLL is the same the input, but with I/O named and grouped.

%% Sizes and placeholders
% Define number of IQCs
N = size(PSICOLL,1);

% Definition of I/O sizes of DELTA
nw = length(P.InputGroup.Uncertain);
nv = length(P.OutputGroup.Uncertain);

% Input names for PSI
PSIInputName = [P.OutputName(P.OutputGroup.Uncertain);...
    P.InputName(P.InputGroup.Uncertain)];

% Placeholder for outputnames
PSIOutputName = cell(N,1);
PSIvOutputName = cell(N,1);
PSIwOutputName = cell(N,1);

%% Group and name I/O channels
for k = 1:N
    % Copy PSIInputName into PSIOutputName{k,1}
    PSIOutputName{k,1} = PSIInputName;

    % Append F(k) to end of PSIOutputName{k,1}
    for kk = 1:nv+nw
        PSIOutputName{k,1}{kk,1} = [PSIOutputName{k,1}{kk,1},'F',num2str(k)];
    end
    
    % Set properties for PSItemp
    PSItemp = PSICOLL{k,1};
    set(PSItemp,'InputName',PSIInputName,...
        'OutputName',PSIOutputName{k,1},...
        'InputGroup',struct('Uncertain',1:length(PSItemp.InputName)),...
        'OutputGroup',...
        struct(['UncertainF',num2str(k)],1:length(PSItemp.OutputName),...
        'Uncertainv',1:nv,...
        'Uncertainw',nv+1:nv+nw));

    % Split PSIOutputName into 'v' and 'w' parts
    PSIvOutputName{k,1} = PSItemp.OutputName(1:nv);
    PSIwOutputName{k,1} = PSItemp.OutputName(nv+1:nv+nw);

    % Copy PSItemp into PSICOLL{k,1}
    PSICOLL{k,1} = PSItemp;
end

%% Form interconnection of all filters within PSICOLL
PSIICT = ss(connect(PSICOLL{:},PSIInputName,...
    [vertcat(PSIvOutputName{:});vertcat(PSIwOutputName{:})]));

% Minimal realization of PSIICT
fprintf('\n - Forming minimal realization of plant and IQC interconnection.\n');
PSIICTmin = minreal(PSIICT);

%% Form interconnection of P and PSICOLL
PICT = connect(P,PSIICTmin,P.InputName,...
    [PSIICTmin.OutputName;...
    P.OutputName([P.OutputGroup.Measurement,P.OutputGroup.ToBeEstimated])]);
end
